@@ -15,17 +15,17 @@
  * limitations under the License.
  */
 
-/* $Id: TIFFDirectory.java 496559 2007-01-16 01:10:29Z cam $ */
+/* $Id: TIFFDirectory.java 607082 2007-12-27 16:21:37Z jeremias $ */
 
 package org.apache.xmlgraphics.image.codec.tiff;
 
 import java.io.IOException;
 import java.io.Serializable;
-import java.util.Iterator;
-import java.util.HashMap;
-import java.util.Map;
-import java.util.List;
 import java.util.ArrayList;
+import java.util.HashMap;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
 
 import org.apache.xmlgraphics.image.codec.util.SeekableStream;
 
@@ -49,7 +49,7 @@ import org.apache.xmlgraphics.image.codec.util.SeekableStream;
  * be removed or changed in future releases of JAI.</b>
  *
  * @see TIFFField
- * @version $Id: TIFFDirectory.java 496559 2007-01-16 01:10:29Z cam $
+ * @version $Id: TIFFDirectory.java 607082 2007-12-27 16:21:37Z jeremias $
  */
 public class TIFFDirectory extends Object implements Serializable {
 
@@ -124,6 +124,10 @@ public class TIFFDirectory extends Object implements Serializable {
 
             ifd_offset = readUnsignedInt(stream);
         }
+        if (ifd_offset == 0L) {
+            throw new
+               IllegalArgumentException("TIFFDirectory3");
+        }
 
         stream.seek(ifd_offset);
         initialize(stream);
