@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: TIFFImage.java 496556 2007-01-16 00:59:48Z cam $ */
+/* $Id: TIFFImage.java 582858 2007-10-08 16:09:50Z jeremias $ */
 
 package org.apache.xmlgraphics.image.codec.tiff;
 
@@ -327,9 +327,15 @@ public class TIFFImage extends AbstractRed {
             compression = compField == null ? COMP_NONE : compField.getAsInt(0);
 
             // Get the photometric interpretation.
-            int photometricType = (int)dir.getFieldAsLong(
-                                                          TIFFImageDecoder.TIFF_PHOTOMETRIC_INTERPRETATION);
-
+            int photometricType;
+            TIFFField photometricTypeField = dir.getField(
+                    TIFFImageDecoder.TIFF_PHOTOMETRIC_INTERPRETATION);
+            if (photometricTypeField == null) {
+                photometricType = 0; // White is zero
+            } else {
+                photometricType = photometricTypeField.getAsInt(0);
+            }
+            
             // Determine which kind of image we are dealing with.
             imageType = TYPE_UNSUPPORTED;
             switch(photometricType) {
