@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PSGraphics2D.java 548990 2007-06-20 08:28:52Z jeremias $ */
+/* $Id: PSGraphics2D.java 603586 2007-12-12 12:13:57Z acumiskey $ */
 
 package org.apache.xmlgraphics.java2d.ps;
 
@@ -37,7 +37,6 @@ import java.awt.Rectangle;
 import java.awt.Shape;
 import java.awt.Stroke;
 import java.awt.TexturePaint;
-import java.awt.color.ColorSpace;
 import java.awt.geom.AffineTransform;
 import java.awt.geom.PathIterator;
 import java.awt.image.BufferedImage;
@@ -48,6 +47,8 @@ import java.io.IOException;
 
 import org.apache.xmlgraphics.java2d.AbstractGraphics2D;
 import org.apache.xmlgraphics.java2d.GraphicContext;
+import org.apache.xmlgraphics.java2d.StrokingTextHandler;
+import org.apache.xmlgraphics.java2d.TextHandler;
 import org.apache.xmlgraphics.ps.PSGenerator;
 import org.apache.xmlgraphics.ps.PSImageUtils;
 
@@ -57,7 +58,7 @@ import org.apache.xmlgraphics.ps.PSImageUtils;
  * code.
  *
  * @author <a href="mailto:keiron@aftexsw.com">Keiron Liddle</a>
- * @version $Id: PSGraphics2D.java 548990 2007-06-20 08:28:52Z jeremias $
+ * @version $Id: PSGraphics2D.java 603586 2007-12-12 12:13:57Z acumiskey $
  * @see org.apache.xmlgraphics.java2d.AbstractGraphics2D
  */
 public class PSGraphics2D extends AbstractGraphics2D {
@@ -66,6 +67,8 @@ public class PSGraphics2D extends AbstractGraphics2D {
     
     private static final boolean DEBUG = false;
     
+    protected PSGraphics2D rootG2D;
+    
     /** the PostScript generator being created */
     protected PSGenerator gen;
 
@@ -109,6 +112,7 @@ public class PSGraphics2D extends AbstractGraphics2D {
     public PSGraphics2D(PSGraphics2D g) {
         super(g);
 
+        this.rootG2D = (g.rootG2D != null ? g.rootG2D : g);
         setPSGenerator(g.gen);
         this.clippingDisabled = g.clippingDisabled;
         this.fallbackTextHandler = g.fallbackTextHandler;
@@ -149,7 +153,7 @@ public class PSGraphics2D extends AbstractGraphics2D {
     }
 
     /**
-     * Sets a custom TextHandler implementation that is reponsible for painting text. The default
+     * Sets a custom TextHandler implementation that is responsible for painting text. The default
      * TextHandler paints all text as shapes. A custom implementation can implement text painting
      * using text painting operators.
      * @param handler the custom TextHandler implementation
@@ -171,6 +175,7 @@ public class PSGraphics2D extends AbstractGraphics2D {
      * this graphics context.
      */
     public Graphics create() {
+        preparePainting();
         return new PSGraphics2D(this);
     }
 
@@ -189,6 +194,9 @@ public class PSGraphics2D extends AbstractGraphics2D {
      */
     public void preparePainting() {
         //nop, used by AbstractPSDocumentGraphics2D
+        if (rootG2D != null) {
+            rootG2D.preparePainting();
+        }
     }
 
     /**
